
From: Zachary Amsden <zach@vmware.com>

i386 arch cleanup.  Introduce the serialize macro to serialize processor
state.  Why the microcode update needs it I am not quite sure, since wrmsr()
is already a serializing instruction, but it is a microcode update, so I will
keep the semantic the same, since this could be a timing workaround.  As far
as I can tell, this has always been there since the original microcode update
source.

Signed-off-by: Zachary Amsden <zach@vmware.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/i386/kernel/microcode.c |    7 +++++--
 include/asm-i386/processor.h |    5 +++++
 2 files changed, 10 insertions(+), 2 deletions(-)

diff -puN arch/i386/kernel/microcode.c~i386-arch-cleanup-seralize-msr arch/i386/kernel/microcode.c
--- devel/arch/i386/kernel/microcode.c~i386-arch-cleanup-seralize-msr	2005-07-30 11:15:44.000000000 -0700
+++ devel-akpm/arch/i386/kernel/microcode.c	2005-07-30 11:15:44.000000000 -0700
@@ -164,7 +164,8 @@ static void collect_cpu_info (void *unus
 	}
 
 	wrmsr(MSR_IA32_UCODE_REV, 0, 0);
-	__asm__ __volatile__ ("cpuid" : : : "ax", "bx", "cx", "dx");
+	/* see notes above for revision 1.07.  Apparent chip bug */
+	serialize_cpu();
 	/* get the current revision from MSR 0x8B */
 	rdmsr(MSR_IA32_UCODE_REV, val[0], uci->rev);
 	pr_debug("microcode: collect_cpu_info : sig=0x%x, pf=0x%x, rev=0x%x\n",
@@ -377,7 +378,9 @@ static void do_update_one (void * unused
 		(unsigned long) uci->mc->bits >> 16 >> 16);
 	wrmsr(MSR_IA32_UCODE_REV, 0, 0);
 
-	__asm__ __volatile__ ("cpuid" : : : "ax", "bx", "cx", "dx");
+	/* see notes above for revision 1.07.  Apparent chip bug */
+	serialize_cpu();
+
 	/* get the current revision from MSR 0x8B */
 	rdmsr(MSR_IA32_UCODE_REV, val[0], val[1]);
 
diff -puN include/asm-i386/processor.h~i386-arch-cleanup-seralize-msr include/asm-i386/processor.h
--- devel/include/asm-i386/processor.h~i386-arch-cleanup-seralize-msr	2005-07-30 11:15:44.000000000 -0700
+++ devel-akpm/include/asm-i386/processor.h	2005-07-30 11:15:44.000000000 -0700
@@ -277,6 +277,11 @@ static inline void clear_in_cr4 (unsigne
 	outb((data), 0x23); \
 } while (0)
 
+static inline void serialize_cpu(void)
+{
+	 __asm__ __volatile__ ("cpuid" : : : "ax", "bx", "cx", "dx");
+}
+
 static inline void __monitor(const void *eax, unsigned long ecx,
 		unsigned long edx)
 {
_
